/* This file was generated by syncqt. */
#ifndef QT_QTPOSITIONING_VERSION_H
#define QT_QTPOSITIONING_VERSION_H

#define QTPOSITIONING_VERSION_STR "6.11.0"

#define QTPOSITIONING_VERSION 0x0601100

#endif // QT_QTPOSITIONING_VERSION_H
